from json import load

from coala_json.reporters.ResultReporter import ResultReporter


class HtmlReporter(ResultReporter):
    """
    Contain methods to produce Html test report from coala-json
    coala_json is coala output in json format, usually produced by
    running coala with --json option
    """

    def to_output(self):
        """
        Convert coala-json output to html table report

        :return: html table report
        """

        output_json = load(self.coala_json)['results']
        table = ('<html><head><title>Test Results Report</title></head><body>'
                 '<table border="1"><h2>Test Result Reports</h2><tr>'
                 '<th>Line</th><th>Column</th><th>Severity</th>'
                 '<th>Message</th><th>Origin</th><th>RuleID</th>'
                 '<th>File</th></tr>')
        for problems in output_json.values():
            for problem in problems:
                line = self.loader.extract_affected_line(problem)
                col = self.loader.extract_affected_column(problem)
                severity = self.loader.extract_severity(problem)
                message = self.loader.extract_message(problem)
                origin = self.loader.extract_origin(problem)
                ruleid = self.loader.extract_error_code(message)
                file = self.loader.extract_file(problem)
                table += ('<tr><td>{}</td><td>{}</td><td>{}</td>'
                          '<td>{}</td><td>{}</td><td>{}</td><td>{}</td>'
                          '</tr>'.format(line, col, severity, message, origin,
                                         ruleid, file))
        table += '</table></body></html>\n'
        return table



import re

from coala_json.loader.JsonLoader import JsonLoader


class coalaJsonLoader(JsonLoader):
    """
    Contains method to extract data from coala-json
    """

    @staticmethod
    def sanitize(error_message):
        """
        Change HTML characters to character entity reference so that
        they are not mixed with XML tags

        :param: string that contains HTML characters
        :return: string with character entity references
        """
        mapping = {'\"': '&quot;', '<': '&lt;', '>': '&gt;'}
        for k, v in mapping.items():
            error_message = error_message.replace(k, v)
        return error_message

    @staticmethod
    def extract_error_code(error_message):
        """
        Return error code from error message or an empty string if
        no error code is found
        """
        try:
            pattern = re.search('[A-Z][0-9]{3,4}', error_message).group()
            return ' {}'.format(pattern)
        except AttributeError:
            return ''

    @staticmethod
    def extract_message(problem):
        return coalaJsonLoader.sanitize(problem['message'])

    @staticmethod
    def extract_raw_message(problem):
        return problem['message']

    @staticmethod
    def extract_affected_line(problem):
        if problem['affected_code']:
            return problem['affected_code'][0]['end']['line']

    @staticmethod
    def extract_affected_column(problem):
        if problem['affected_code']:
            return problem['affected_code'][0]['end']['column']

    @staticmethod
    def extract_file(problem):
        if problem['affected_code']:
            return problem['affected_code'][0]['file']

    @staticmethod
    def extract_origin(problem):
        return problem['origin'].split(" ")[0]

    @staticmethod
    def extract_severity(problem):
        return problem['severity']

    @staticmethod
    def extract_errors(problems):
        return len(problems)



import os
import argparse
import sys

from coala_json.reporters.ReporterFactory import ReporterFactory
from coala_json.loader.coalaJsonLoader import coalaJsonLoader


def get_path(filepath):
    return os.path.join(os.getcwd(), filepath)


def create_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('--junit', const=True, action='store_const',
                        help='mode in which coala will produce a JUnit report')
    parser.add_argument('--checkstyle', const=True, action='store_const',
                        help='mode in which coala will produce a'
                             ' Checkstyle report')
    parser.add_argument('--tap', const=True, action='store_const',
                        help='mode in which coala will produce a TAP report')
    parser.add_argument('--table', const=True, action='store_const',
                        help='mode in which coala will produce a HTML table'
                             ' report')
    parser.add_argument('-f', '--input', help='path of the json input file')
    parser.add_argument('-o', '--output', help='path of output report file. '
                                               'If nothing is specified then '
                                               'coala-json will print the '
                                               'output to the stdout')
    return parser


def main():
    parser = create_parser()
    args = parser.parse_args()
    produce_report(parser, args)


def produce_report(parser, args):
    if not args.input:
        parser.error("Please specify a 'coala-json' input file")

    with open(get_path(args.input)) as input_file:
        factory = ReporterFactory(coalaJsonLoader(), parser, input_file, args)
        reporter = factory.get_reporter()
        output = reporter.to_output()

    if args.output:
        with open(args.output, 'w+') as report:
            report.write(output)
    else:
        sys.stdout.write(output)


if __name__ == '__main__':
    sys.exit(main())



"""coala-json holds utilities to manipulate coala JSON output."""


from os.path import join, dirname


VERSION_FILE = join(dirname(__file__), 'VERSION')


def get_version():
    with open(VERSION_FILE, 'r') as ver:
        return ver.readline().strip()


VERSION = get_version()
__version__ = VERSION



from coala_json.reporters.CheckstyleReporter import CheckstyleReporter
from coala_json.reporters.HtmlReporter import HtmlReporter
from coala_json.reporters.JunitReporter import JunitReporter
from coala_json.reporters.TapReporter import TapReporter


class ReporterFactory:
    """
    Use to generate instances of reporters
    """

    def __init__(self, loader, parser, input_file, args):
        self.loader = loader
        self.parser = parser
        self.input_file = input_file
        self.args = args

    def get_reporter(self):
        """
        Get reporter according to args if available else returns error.
        """
        if self.args.checkstyle:
            return CheckstyleReporter(self.loader, self.input_file)

        elif self.args.junit:
            return JunitReporter(self.loader, self.input_file)

        elif self.args.tap:
            return TapReporter(self.loader, self.input_file)

        elif self.args.table:
            return HtmlReporter(self.loader, self.input_file)

        else:
            return self.parser.error("Please check if a single output mode is"
                                     " specified correctly")



import datetime
from json import load

from coala_json.reporters.ResultReporter import ResultReporter


class JunitReporter(ResultReporter):
    """
    Contain methods to produce Junit test report from coala-json
    coala_json is coala output in json format, usually produced by
    running coala with --json option
    """

    def to_output(self):
        """
        Convert coala-json output to coala-junit test result report

        :return: junit test result report
        """

        timestamp = datetime.datetime.utcnow().isoformat()
        output_json = load(self.coala_json)['results']
        junit = ''
        for problems in output_json.values():
            for problem in problems:
                message = self.loader.extract_message(problem)
                affected_line = self.loader.extract_affected_line(problem)
                affected_col = self.loader.extract_affected_column(problem)
                testsuite_name = self.loader.extract_file(problem)
                testsuite_package = self.loader.extract_origin(problem)
                failures = self.loader.extract_errors(problems)
                testcase_name = self.loader.extract_error_code(message)
                error_message = "line: {}, Column: {}, {}".format(
                    affected_line, affected_col, message)
                junit += ('<testsuite package="{}" timestamp="{}" tests="{}" '
                          'failures="{}" name="{}">\n<testcase name="{}{}">\n'
                          '<failure message="{}"></failure>\n</testcase>\n'
                          '</testsuite>\n'.format(testsuite_package, timestamp,
                                                  failures, failures,
                                                  testsuite_name,
                                                  testsuite_package,
                                                  testcase_name,
                                                  error_message))
        if junit:
            head = '<?xml version="1.0" encoding="utf-8"?>\n<testsuites>\n'
            output = head + junit + '</testsuites>'
        else:
            output = ('<?xml version="1.0" encoding="utf-8"?>\n<testsuites>\n'
                      '<testsuite timestamp="{}" '
                      'tests="1" failures="0" name="coala">\n'
                      '<testcase time="0" name="None"></testcase>\n'
                      '</testsuite>\n'
                      '</testsuites>'.format(timestamp))
        return output



from json import load

from coala_json.reporters.ResultReporter import ResultReporter


class TapReporter(ResultReporter):
    """
    Contain methods to produce TAP test report from coala-json.
    coala_json is coala output in json format, usually produced by
    running coala with --json option
    """

    def to_output(self):
        """
        Convert coala-json output to coala-TAP test result report

        :return: TAP test result report
        """

        output_json = load(self.coala_json)['results']
        report = ''
        tests = '1'
        for problems in output_json.values():
            for test_no, problem in enumerate(problems):
                file_name = self.loader.extract_file(problem)
                message = self.loader.extract_raw_message(problem)
                error_message = message.replace(':', ' -')
                severity = self.loader.extract_severity(problem)
                affected_line = self.loader.extract_affected_line(problem)
                affected_col = self.loader.extract_affected_column(problem)
                rule_id = self.loader.extract_error_code(message)
                tests = self.loader.extract_errors(problems)
                # severity 0 is for errors related to INFO, the coala-ci build
                # still passes
                status = 'ok' if severity == 0 else 'not ok'
                report += ('{} {} - {}\n  ---\n  message: {}\n  severity: '
                           '{}\n  data:\n    line: {}\n    column: {}\n    '
                           'ruleId:{}\n  ...\n'.format(status, test_no + 1,
                                                       file_name,
                                                       error_message, severity,
                                                       affected_line,
                                                       affected_col, rule_id))
        head = 'TAP version 13\n1..{}\n'.format(tests)
        if report:
            output = head + report
        else:
            output = head + 'ok 1 - coala\n...'
        return output



import json


def _output_to_lsp(output):
    """
    Turn output to diagnstics and completions.
    """
    if output is None:
        return None, None
    output_json = json.loads(output)['results']
    res = []
    fixes = []
    for key, problems in output_json.items():
        section = key
        for problem in problems:
            """
            Transform RESULT_SEVERITY of coala into DiagnosticSeverity of LSP
            coala: INFO = 0, NORMAL = 1, MAJOR = 2
            LSP: Error = 1, Warning = 2, Information = 3, Hint = 4
            """
            severity = 3 - problem['severity']
            message = problem['message']
            origin = problem['origin']
            real_message = '[{}] {}: {}'.format(section, origin, message)

            # TODO Handle results for multiple files
            # and also figure out a way to resolve
            # overlapping patches.

            if problem.get('diffs', None) is not None:
                for file, diff in problem['diffs'].items():
                    fixes.append({
                        'filename': file,
                        'diff': diff,
                    })

            for code in problem['affected_code']:
                """
                Line position and character offset should be zero-based
                according to LSP, but row and column positions of coala
                are None or one-based number.
                coala uses None for convenience. None for column means the
                whole line while None for line means the whole file.
                """
                def convert_offset(x): return x - 1 if x else x
                start_line = convert_offset(code['start']['line'])
                start_char = convert_offset(code['start']['column'])
                end_line = convert_offset(code['end']['line'])
                end_char = convert_offset(code['end']['column'])
                if start_char is None or end_char is None:
                    start_char = 0
                    end_line = start_line + 1
                    end_char = 0
                res.append({
                    'severity': severity,
                    'range': {
                        'start': {
                            'line': start_line,
                            'character': start_char
                        },
                        'end': {
                            'line': end_line,
                            'character': end_char
                        }
                    },
                    'source': 'coala',
                    'message': real_message
                })

    return res, fixes


def output_to_diagnostics(output):
    """
    Turn output to diagnstics.
    """
    return _output_to_lsp(output)[0]



from json import load

from coala_json.reporters.ResultReporter import ResultReporter


class CheckstyleReporter(ResultReporter):
    """
    Contain methods to produce Checkstyle test report from coala-json.
    coala_json is coala output in json format, usually produced by
    running coala with --json option
    """

    def to_output(self):
        """
        Convert coala-json output to coala-checkstyle test result report

        :return: checkstyle test result report
        """
        output_json = load(self.coala_json)['results']
        report = ''
        for problems in output_json.values():
            for problem in problems:
                """
                map RESULT_SEVERITY of coala to Checkstyle severity options
                coala: INFO = 0, NORMAL = 1, MAJOR = 2
                checkstyle options: error, warning, info, ignore
                """
                severity = self.loader.extract_severity(problem)
                severity_map = {0: 'info', 1: 'warning', 2: 'error'}
                message = self.loader.extract_message(problem)
                error_line = self.loader.extract_affected_line(problem)
                error_col = self.loader.extract_affected_column(problem)
                file_name = self.loader.extract_file(problem)
                source = self.loader.extract_origin(problem)
                error_code = self.loader.extract_error_code(message)
                report += ('<file name="{}">\n<error severity="{}" line="{}" '
                           'column="{}" message="{}" source="{} {}">\n</error>'
                           '\n</file>\n'.format(file_name,
                                                severity_map[severity],
                                                error_line, error_col, message,
                                                error_code, source))
        head = ('<?xml version="1.0" encoding="utf-8"?>\n'
                '<checkstyle version="4.3">\n')
        output = head + report + '</checkstyle>\n'
        return output



import re

from coala_json.loader.coalaJsonLoader import coalaJsonLoader


class ResultReporter:
    """
    Every test report class is a sub class of ResultReporter
    """

    def __init__(self, loader: coalaJsonLoader, coala_json):
        self.loader = loader
        self.coala_json = coala_json



#!/usr/bin/env python3

import locale
import os
import platform
import sys

import pip
from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand

try:
    lc = locale.getlocale()
    pf = platform.system()
    if pf != 'Windows' and lc == (None, None):
        locale.setlocale(locale.LC_ALL, 'C.UTF-8')
except (ValueError, UnicodeError, locale.Error):
    locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')

VERSION = '0.1.1'
DESCRIPTION = (
    'A collection of coala json utilities'
)
DEPENDENCY_LINKS = []

KEYWORDS = [
    'code-analysis',
    'lint',
]


# from http://pypi.python.org/pypi?%3Aaction=list_classifiers

CLASSIFIERS = [
    'Development Status :: 4 - Beta',

    'Environment :: Console',
    'Environment :: MacOS X',
    'Environment :: Win32 (MS Windows)',

    'Intended Audience :: Science/Research',
    'Intended Audience :: Developers',

    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',

    'Programming Language :: Python :: Implementation :: CPython',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3 :: Only',

    'Topic :: Scientific/Engineering :: Information Analysis',
    'Topic :: Software Development :: Quality Assurance',
    'Topic :: Text Processing :: Linguistic',

]

SETUP_COMMANDS = {}


def set_python_path(path):
    if 'PYTHONPATH' in os.environ:
        user_paths = os.environ['PYTHONPATH'].split(os.pathsep)
        user_paths.insert(0, path)
        os.environ['PYTHONPATH'] = os.pathsep.join(user_paths)
    else:
        os.environ['PYTHONPATH'] = path


class PyTestCommand(TestCommand):
    """
    From https://pytest.org/latest/goodpractices.html
    """
    user_options = [('pytest-args=', 'a', 'Arguments to pass to py.test')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


SETUP_COMMANDS['test'] = PyTestCommand


__dir__ = os.path.dirname(__file__)


from distutils.version import LooseVersion  # noqa (late-import)


class PEP440Version(LooseVersion):
    """
    Basic PEP440 version with a few features.

    Uses the same version semantics as LooseVersion,
    with the addition that a ``v`` prefix is allowed
    in the version as required by PEP 440.

    vstring may be a list, tuple or string.

    v_prefix indicates whether output of the version
    should include a v prefix.

    v_prefix is auto-detected by default.
    Set to False to remove if present, or True to add if missing.
    """

    def __init__(self, vstring=None, v_prefix=None):
        self._v_prefix = v_prefix

        if isinstance(vstring, (list, tuple)):
            type_ = type(vstring)
            vstring = '.'.join(str(i) for i in vstring)
        else:
            type_ = list

        vstring = vstring.strip()

        if vstring.startswith('v'):
            vstring = vstring[1:]
            if vstring.startswith('!'):
                raise ValueError('Invalid use of epoch')
            if v_prefix is not False:
                self._v_prefix = True

        # Can not use super(..) on Python 2.7
        LooseVersion.__init__(self, vstring)
        if self._v_prefix:
            self.vstring = 'v' + self.vstring
        if len(self.version) > 1 and self.version[1] == '!':
            self._epoch = self.version[0]
            if not isinstance(self._epoch, int) or len(self.version) < 3:
                raise ValueError('Invalid use of epoch')

        # Normalise to lower case
        self.version = [
            x if isinstance(x, int) else x.lower() for x in self.version
            if x not in ('-', '_')]

        if self.version[-1] != '*' and not isinstance(self.version[-1], int):
            self.version += (0, )

        if type_ is tuple:
            self.version = tuple(self.version)

        self._final = None
        self._previous = None

    def __repr__(self):
        return "%s('%s')" % (self.__class__.__name__, str(self))

    @property
    def is_dev(self):
        return any(part == 'dev' for part in self.version)

    @property
    def has_epoch(self):
        return any(part == '!' for part in self.version)

    @property
    def final(self):
        """
        Provide only the final component of the version.

        A new instance is return if this instance is not final.
        """
        if self.has_epoch:
            raise NotImplementedError

        if self._final is not None:
            return self._final

        for i, part in enumerate(self.version):
            if not isinstance(part, int):
                final = self.version[:i]
                break
        else:
            self._final = self
            return self

        self._final = PEP440Version(final, self._v_prefix)

        return self._final

    @property
    def is_final(self):
        return self.final == self

    @property
    def is_zero(self):
        return all(part == 0 for part in self.version)

    _zero_message = 'version prior to 0.0 can not exist'

    def _estimate_previous(self):
        """
        Return a new version calculated to be the previous version.

        Currently only handles when the current instance is a final version.

        To really get the previous for 1.0.0, we need to consult PyPi,
        git tags, or some other source of all released versions,
        to find the highest patch release in the prior minor release, or
        highest minor releases if there were no patch releases in the
        last minor release, etc.

        As a result, currently this assumes that release x.(x-1).0 exists
        in that instance.
        """
        if self._previous:
            return self._previous

        assert self.is_final, '%r is not final' % self

        if self.is_zero:
            raise ValueError(self._zero_message)

        previous = self._decrement(self.version)
        self._previous = PEP440Version(previous, self._v_prefix)
        return self._previous

    @staticmethod
    def _decrement(version):
        pos = len(version) - 1

        # Look for non-zero int part
        while pos != 0 and not (isinstance(version[pos], int) and version[pos]):
            pos -= 1

        previous = []
        if pos:
            previous = version[:pos]

        previous += (version[pos] - 1, )

        if len(previous) == len(version):
            return previous

        remaining = version[pos + 1:-1]

        previous += tuple(
            0 if isinstance(i, int) else i for i in remaining)

        previous += ('*', )

        return previous


# This is an unfortunately needed comment

def egg_name_to_requirement(name):
    name = name.strip()
    parts = name.split('-')

    # The first part may be v or v0, which would be considered a version
    # if processed in the following loop.
    name_parts = [parts[0]]
    # Pre-releases may contain a '-' and be alpha only, so we must
    # parse from the second part to find the first version-like part.
    for part in parts[1:]:
        version = PEP440Version(part)
        if isinstance(version.version[0], int):
            break
        name_parts.append(part)

    version_parts = parts[len(name_parts):]

    if not version_parts:
        return name

    name = '-'.join(name_parts)

    version = PEP440Version('-'.join(version_parts))

    # Assume that alpha, beta, pre, post & final releases
    # are in PyPi so setuptools can find it.
    if not version.is_dev:
        return name + '==' + str(version)

    # setuptools fails if a version is given with any specifier such as
    # `==`, `=~`, `>`, if the version is not in PyPi.

    # For development releases, which will not usually be PyPi,
    # setuptools will typically fail.

    # So we estimate a previous release that should exist in PyPi,
    # by decrementing the lowest final version part, and use version
    # specifier `>` so that the installed package from VCS will have a
    # version acceptable to the requirement.

    # With major and minor releases, the previous version must be guessed.
    # If the version was `2.1.0`, the previous_version will be literally
    # `2.0.*` as it assumes that a prior minor release occurred and used
    # the same versioning convention.
    previous_version = version.final._estimate_previous()

    if previous_version.is_zero:
        raise ValueError(
            'Version %s could not be decremented' % version)

    return name + '>' + str(previous_version)


def read_requirements(filename):
    """
    Parse a requirements file.

    Accepts vcs+ links, and places the URL into
    `DEPENDENCY_LINKS`.

    :return: list of str for each package
    """
    data = []
    filename = os.path.join(__dir__, filename)
    with open(filename) as requirements:
        required = requirements.read().splitlines()
        for line in required:
            if not line or line.startswith('#'):
                continue

            if '+' in line[:4]:
                repo_link, egg_name = line.split('#egg=')
                if not egg_name:
                    raise ValueError('Unknown requirement: {0}'
                                     .format(line))

                DEPENDENCY_LINKS.append(line)

                line = egg_name_to_requirement(egg_name)

            data.append(line)

    return data


test_required = read_requirements('test-requirements.txt')

filename = os.path.join(__dir__, 'README.rst')
with open(filename) as readme:
    long_description = readme.read()

extras_require = None
EXTRAS_REQUIRE = {}
data_files = None

if extras_require:
    EXTRAS_REQUIRE = extras_require
SETUP_COMMANDS.update({
})

if __name__ == '__main__':
    pip_version = LooseVersion(pip.__version__)
    if pip_version < LooseVersion('9.0.1'):
        raise RuntimeError('Version of pip is less than 9.0.1. '
                           'Consider upgrading pip to pip~=9.0.1')
    setup(name='coala-json',
          version=VERSION,
          description=DESCRIPTION,
          author='The coala developers',
          author_email='coala-devel@googlegroups.com',
          maintainer='Lasse Schuirmann, Fabian Neuschmidt, Mischa Kr\xfcger',
          maintainer_email=('lasse.schuirmann@gmail.com, '
                            'fabian@neuschmidt.de, '
                            'makman@alice.de'),
          url='https://gitlab.com/coala/coala-json',
          platforms='any',
          packages=find_packages(exclude=('build.*', 'tests', 'tests.*')),
          extras_require=EXTRAS_REQUIRE,
          tests_require=test_required,
          dependency_links=DEPENDENCY_LINKS,
          package_data={'coala_json': ['VERSION']},
          license='MIT',
          keywords=KEYWORDS,
          data_files=data_files,
          long_description=long_description,
          entry_points={
              'console_scripts': [
                  'coala-json=coala_json.reporters.cli.cli:main',
              ],
          },
          classifiers=CLASSIFIERS,
          cmdclass=SETUP_COMMANDS,
          )
